package com.shcd.ghjg.shedule;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.shcd.ghjg.domain.ControllerRule;
import com.shcd.ghjg.mapper.SuperviseMailMapper;
import com.shcd.ghjg.service.ControllerRuleService;
import com.shcd.ghjg.service.SuperviseMailService;
import com.shcd.ghjg.util.DateUtil;
import com.shcd.ghjg.util.EmailUtil;
import com.shcd.ghjg.vo.Rule;
import com.shcd.ghjg.vo.RuleUser;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.stream.Collectors;

@Component
@Slf4j
public class SuperviseMailSchedule {
    @Autowired
    private SuperviseMailService superviseMailService;

    @Autowired
    private ControllerRuleService controllerRuleService;

    @Autowired
    private SuperviseMailMapper superviseMailMapper;

    @Autowired
    private EmailUtil emailUtil;

    @XxlJob("ruleMessageJob")
//    @PostConstruct
    public ReturnT<String> sendMail(String param) {
//    public void sendMail() {
        List<Rule> allRule = superviseMailMapper.getAllRule();
        ArrayList<Map<String, String>> maps = new ArrayList<>();
        ArrayList<ControllerRule> rules = new ArrayList<>();
        for (Rule rule : allRule) {
            try {
                Map<String, Object> object = null;
                try {
                    object = runSql(rule.getRuleSql(), rule.getParams());
                } catch (Exception e) {
                    log.error("规则出错{}", rule.getRuleName() + rule.getRuleSql() + e.getMessage());
                }
                System.out.println(rule.getRuleSql());
                if (object != null && !object.isEmpty() && "1".equals(rule.getLastState())) {
                    //需要发送异常邮件
                    if (DateUtil.isIn(new Date(), 9, 18)) {
                        //工作时间邮件发生正常频率
                        if (rule.getUpdateDate() == null || rule.getUpdateDate().getTime() + rule.getTaktTimeWork() < new Date().getTime()) {
                            //获取需要发送的内容;
                            ControllerRule controllerRule = new ControllerRule();
                            controllerRule.setId(rule.getId().longValue());
                            controllerRule.setLastState("1");
                            controllerRule.setUpdateTime(new Date());
                            rules.add(controllerRule);
                            makeContent(rule, object, maps);
                        }
                    } else {
                        //非工作时间邮件发送频率降低
                        if (rule.getUpdateDate() == null || rule.getUpdateDate().getTime() + rule.getTaktTimeUnWork() < new Date().getTime()) {
                            //获取发送的模板
                            ControllerRule controllerRule = new ControllerRule();
                            controllerRule.setId(rule.getId().longValue());
                            controllerRule.setLastState("1");
                            controllerRule.setUpdateTime(new Date());
                            rules.add(controllerRule);
                            makeContent(rule, object, maps);
                        }
                    }
                }
                if (object == null && "1".equals(rule.getLastState())) {
                    //发恢复邮件
                    HashMap<String, Object> map = new HashMap<>();
                    map.put("id", rule.getId().toString());
                    String res = rule.getRuleName() + "恢复正常";
                    map.put("res", res);
                    ControllerRule controllerRule = new ControllerRule();
                    controllerRule.setId(rule.getId().longValue());
                    controllerRule.setLastState("0");
                    controllerRule.setUpdateTime(new Date());
                    rules.add(controllerRule);
                    makeContent2(rule, map, maps);
                }
            } catch (Exception e) {
                log.error("出现未知异常{}",e.getMessage());
            }

            //其他情况不发送邮件
        }
        if (!maps.isEmpty()) {
            //获取发送人
            List<RuleUser> allUser = superviseMailMapper.getAllUser();
            //发送邮件
            sendSupervisionMail(allUser, maps);
            //更新规则表
            //superviseMailMapper.updateRuleDate(rules);
            controllerRuleService.updateBatchById(rules);
        }
        return ReturnT.SUCCESS;
    }

    private void makeContent2(Rule rule, HashMap<String, Object> rest, ArrayList<Map<String, String>> maps) {
        HashMap<String, String> map = new HashMap<>();
        map.put("id", rule.getId().toString());
        String res = rest.get("res").toString();
        map.put("res", res);
        map.put("title", rule.getRuleName());
        maps.add(map);
    }

    private static void makeContent(Rule rule, Map<String, Object> object, ArrayList<Map<String, String>> maps) {
        HashMap<String, String> map = new HashMap<>();
        map.put("id", rule.getId().toString());
        String res = rule.getErrMsg();
        for (Map.Entry<String, Object> entry : object.entrySet()) {
            res = res.replace(entry.getKey(), entry.getValue().toString());
        }
        map.put("res", res);
        map.put("title", rule.getRuleName());
        maps.add(map);
    }

    private void sendSupervisionMail(List<RuleUser> allUser, ArrayList<Map<String, String>> maps) {
        for (Map<String, String> map : maps) {
            Map<String, List<RuleUser>> collect = allUser.stream().collect(Collectors.groupingBy(RuleUser::getRuleId));
            List<RuleUser> equal = collect.get(map.get("id"));
            if (equal != null) {
                for (RuleUser ruleUser : equal) {
                    //邮件发送接口
                    String sendEmail = emailUtil.sendEmail(ruleUser.getEmail(), "sys", map.get("res"), map.get("title"));
                    log.info(sendEmail);
                }
            }
        }
    }

    private Map<String, Object> runSql(String ruleSql, String params) {
        if (StringUtils.isNotBlank(params)) {
            String[] split = ruleSql.split("@");
            String[] split1 = params.split("&");
            StringBuilder ruleSqlBuilder = new StringBuilder();
            for (int i = 0; i < split.length; i++) {
                ruleSqlBuilder.append(split[i]).append(split1[i]);
            }
            ruleSql = ruleSqlBuilder.toString();
        }
        return superviseMailMapper.runSql(ruleSql);
    }
}
